@@ -336,9 +336,9 @@ class Album(DataObject, Item):
         )
 
     def _recordings_request_finished(self, document, http, error):
-        self._requests -= 1
         if error:
             self.error_append(http.errorString())
+            self._requests -= 1
             self._finalize_loading(error)
         else:
             for recording in document.get('recordings', []):
@@ -349,11 +349,13 @@ class Album(DataObject, Item):
             offset = document.get('recording-offset', 0)
             next_offset = offset + RECORDING_QUERY_LIMIT
             if next_offset < count:
+                self._requests -= 1
                 self._request_recording_relationships(offset=next_offset)
             else:
                 # Merge separately loaded recording relationships into release node
                 self._merge_release_recording_relationships()
                 self._run_album_metadata_processors()
+                self._requests -= 1
                 self._finalize_loading(error)
 
     def _merge_recording_relationships(self, track_node):
@@ -522,15 +524,30 @@ class Album(DataObject, Item):
         self._after_load_callbacks = []
         if self.item.isSelected():
             self.tagger.window.refresh_metadatabox()
+            self.tagger.window.cover_art_box.update_metadata()
 
     def _finalize_loading(self, error):
+        if self.loaded:
+            # This is not supposed to happen, _finalize_loading should only
+            # be called once after all requests finished.
+            import inspect
+            stack = inspect.stack()
+            args = [self]
+            msg = 'Album._finalize_loading called for already loaded album %r'
+            if len(stack) > 1:
+                f = stack[1]
+                msg += ' at %s:%d in %s'
+                args.extend((f.filename, f.lineno, f.function))
+            log.warning(msg, *args)
+            return
+
         if error:
             self.metadata.clear()
             self.status = AlbumStatus.ERROR
-            del self._new_metadata
-            del self._new_tracks
             self.update()
             if not self._requests:
+                del self._new_metadata
+                del self._new_tracks
                 self.loaded = True
                 for func, always in self._after_load_callbacks:
                     if always:
