@@ -4,13 +4,14 @@
 #
 # Copyright (C) 2006-2008, 2012 Lukáš Lalinský
 # Copyright (C) 2008 Hendrik van Antwerpen
-# Copyright (C) 2008-2010, 2014-2015, 2018-2020 Philipp Wolfer
+# Copyright (C) 2008-2010, 2014-2015, 2018-2022 Philipp Wolfer
 # Copyright (C) 2012-2013 Michael Wiencek
 # Copyright (C) 2012-2014 Wieland Hoffmann
 # Copyright (C) 2013 Calvin Walton
-# Copyright (C) 2013-2014, 2017-2019 Laurent Monin
+# Copyright (C) 2013-2014, 2017-2022 Laurent Monin
 # Copyright (C) 2016-2018 Sambhav Kothari
 # Copyright (C) 2017 Ville Skyttä
+# Copyright (C) 2022 Marcin Szalowicz
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -38,19 +39,14 @@ import mutagen.oggspeex
 import mutagen.oggtheora
 import mutagen.oggvorbis
 
-from picard import (
-    config,
-    log,
-)
+from picard import log
+from picard.config import get_config
 from picard.coverart.image import (
     CoverArtImageError,
     TagCoverArtImage,
 )
+from picard.coverart.utils import types_from_id3
 from picard.file import File
-from picard.formats.id3 import (
-    image_type_as_id3_num,
-    types_from_id3,
-)
 from picard.formats.util import guess_format
 from picard.metadata import Metadata
 from picard.util import (
@@ -61,6 +57,7 @@ from picard.util import (
 
 FLAC_MAX_BLOCK_SIZE = 2 ** 24 - 1  # FLAC block size is limited to a 24 bit integer
 INVALID_CHARS = re.compile('([^\x20-\x7d]|=)')
+UNSUPPORTED_TAGS = {'r128_album_gain', 'r128_track_gain'}
 
 
 def sanitize_key(key):
@@ -77,7 +74,7 @@ def is_valid_key(key):
     Valid characters for Vorbis comment field names are
     ASCII 0x20 through 0x7D, 0x3D ('=') excluded.
     """
-    return INVALID_CHARS.search(key) is None
+    return key and INVALID_CHARS.search(key) is None
 
 
 def flac_sort_pics_after_tags(metadata_blocks):
@@ -106,6 +103,18 @@ def flac_sort_pics_after_tags(metadata_blocks):
         metadata_blocks.insert(tagindex, pic)
 
 
+def flac_remove_empty_seektable(file):
+    """Removes an existing but empty seektable from the Flac file.
+
+    Some software has issues with files that contain an empty seek table. Since
+    no seektable is also valid, remove it.
+    """
+    seektable = file.seektable
+    if seektable and not seektable.seekpoints:
+        file.metadata_blocks = [b for b in file.metadata_blocks if b != file.seektable]
+        file.seektable = None
+
+
 class VCommentFile(File):
 
     """Generic VComment-based file."""
@@ -118,10 +127,11 @@ class VCommentFile(File):
         "musicbrainz_trackid": "musicbrainz_recordingid",
         "waveformatextensible_channel_mask": "~waveformatextensible_channel_mask",
     }
-    __rtranslate = dict([(v, k) for k, v in __translate.items()])
+    __rtranslate = {v: k for k, v in __translate.items()}
 
     def _load(self, filename):
         log.debug("Loading file %r", filename)
+        config = get_config()
         file = self._File(encode_filename(filename))
         file.tags = file.tags or {}
         metadata = Metadata()
@@ -179,6 +189,7 @@ class VCommentFile(File):
                             comment=image.desc,
                             support_types=True,
                             data=image.data,
+                            id3_type=image.type
                         )
                     except (CoverArtImageError, TypeError, ValueError, mutagen.flac.error) as e:
                         log.error('Cannot load image from %r: %s' % (filename, e))
@@ -198,6 +209,7 @@ class VCommentFile(File):
                         comment=image.desc,
                         support_types=True,
                         data=image.data,
+                        id3_type=image.type
                     )
                 except CoverArtImageError as e:
                     log.error('Cannot load image from %r: %s' % (filename, e))
@@ -226,19 +238,30 @@ class VCommentFile(File):
     def _save(self, filename, metadata):
         """Save metadata to the file."""
         log.debug("Saving file %r", filename)
+        config = get_config()
         is_flac = self._File == mutagen.flac.FLAC
         file = self._File(encode_filename(filename))
         if file.tags is None:
             file.add_tags()
         if config.setting["clear_existing_tags"]:
-            channel_mask = file.tags.get('waveformatextensible_channel_mask', None)
+            preserve_tags = ['waveformatextensible_channel_mask']
+            if not is_flac and config.setting["preserve_images"]:
+                preserve_tags.append('metadata_block_picture')
+                preserve_tags.append('coverart')
+            preserved_values = {}
+            for name in preserve_tags:
+                if name in file.tags and file.tags[name]:
+                    preserved_values[name] = file.tags[name]
             file.tags.clear()
-            if channel_mask:
-                file.tags['waveformatextensible_channel_mask'] = channel_mask
+            for name, value in preserved_values.items():
+                file.tags[name] = value
         images_to_save = list(metadata.images.to_be_saved_to_tags())
-        if is_flac and (config.setting["clear_existing_tags"] or images_to_save):
+        if is_flac and (images_to_save
+                or (config.setting["clear_existing_tags"]
+                    and not config.setting["preserve_images"])):
             file.clear_pictures()
         tags = {}
+
         for name, value in metadata.items():
             if name == '~rating':
                 # Save rating according to http://code.google.com/p/quodlibet/wiki/Specs_VorbisComments
@@ -280,7 +303,7 @@ class VCommentFile(File):
             picture.desc = image.comment
             picture.width = image.width
             picture.height = image.height
-            picture.type = image_type_as_id3_num(image.maintype)
+            picture.type = image.id3_type
             if is_flac:
                 # See https://xiph.org/flac/format.html#metadata_block_picture
                 expected_block_size = (8 * 4 + len(picture.data)
@@ -301,6 +324,8 @@ class VCommentFile(File):
 
         if is_flac:
             flac_sort_pics_after_tags(file.metadata_blocks)
+            if config.setting["fix_missing_seekpoints_flac"]:
+                flac_remove_empty_seektable(file)
 
         kwargs = {}
         if is_flac and config.setting["remove_id3_from_flac"]:
@@ -314,8 +339,8 @@ class VCommentFile(File):
         """Remove the tags from the file that were deleted in the UI"""
         for tag in metadata.deleted_tags:
             real_name = self._get_tag_name(tag)
-            if real_name and real_name in tags:
-                if real_name in ('performer', 'comment'):
+            if is_valid_key(real_name) and real_name in tags:
+                if real_name in {'performer', 'comment'}:
                     parts = tag.split(':', 1)
                     if len(parts) == 2:
                         tag_type_regex = re.compile(r"\(%s\)$" % re.escape(parts[1]))
@@ -327,13 +352,14 @@ class VCommentFile(File):
                             existing_tags.remove(item)
                     tags[real_name] = existing_tags
                 else:
-                    if tag in ('totaldiscs', 'totaltracks') and tag in tags:
+                    if tag in {'totaldiscs', 'totaltracks'} and tag in tags:
                         # both tag and real_name are to be deleted in this case
                         del tags[tag]
                     del tags[real_name]
 
     def _get_tag_name(self, name):
         if name == '~rating':
+            config = get_config()
             if config.setting['rating_user_email']:
                 return 'rating:%s' % config.setting['rating_user_email']
             else:
@@ -357,8 +383,7 @@ class VCommentFile(File):
 
     @classmethod
     def supports_tag(cls, name):
-        unsupported_tags = ['r128_album_gain', 'r128_track_gain']
-        return (bool(name) and name not in unsupported_tags
+        return (bool(name) and name not in UNSUPPORTED_TAGS
                 and (is_valid_key(name)
                     or name.startswith('comment:')
                     or name.startswith('lyrics:')
