@@ -4,7 +4,7 @@
 #
 # Copyright (C) 2006-2007, 2011 Lukáš Lalinský
 # Copyright (C) 2009 Carlin Mangar
-# Copyright (C) 2009, 2018-2021 Philipp Wolfer
+# Copyright (C) 2009, 2018-2022 Philipp Wolfer
 # Copyright (C) 2011-2013 Michael Wiencek
 # Copyright (C) 2012 Chad Wilson
 # Copyright (C) 2012-2014 Wieland Hoffmann
@@ -52,16 +52,24 @@ from picard.const import MAX_COVERS_TO_STACK
 from picard.coverart.image import (
     CoverArtImage,
     CoverArtImageError,
+    CoverArtImageIOError,
 )
 from picard.file import File
 from picard.track import Track
-from picard.util import imageinfo
+from picard.util import (
+    imageinfo,
+    normpath,
+)
 from picard.util.lrucache import LRUCache
 
 from picard.ui.item import FileListItem
 from picard.ui.widgets import ActiveLabel
 
 
+THUMBNAIL_WIDTH = 128
+COVERART_WIDTH = THUMBNAIL_WIDTH - 7
+
+
 class CoverArtThumbnail(ActiveLabel):
     image_dropped = QtCore.pyqtSignal(QtCore.QUrl, bytes)
 
@@ -69,21 +77,50 @@ class CoverArtThumbnail(ActiveLabel):
         super().__init__(active, drops, *args, **kwargs)
         self.data = None
         self.has_common_images = None
-        self.shadow = QtGui.QPixmap(":/images/CoverArtShadow.png")
-        self.pixel_ratio = self.tagger.primaryScreen().devicePixelRatio()
-        w, h = self.scaled(128, 128)
-        self.shadow = self.shadow.scaled(w, h, QtCore.Qt.KeepAspectRatio, QtCore.Qt.SmoothTransformation)
-        self.shadow.setDevicePixelRatio(self.pixel_ratio)
         self.release = None
+        window_handle = self.window().windowHandle()
+        if window_handle:
+            self.pixel_ratio = window_handle.screen().devicePixelRatio()
+            window_handle.screenChanged.connect(self.screen_changed)
+        else:
+            self.pixel_ratio = self.tagger.primaryScreen().devicePixelRatio()
+        self._pixmap_cache = pixmap_cache
+        self._update_default_pixmaps()
         self.setPixmap(self.shadow)
-        self.setAlignment(QtCore.Qt.AlignTop | QtCore.Qt.AlignHCenter)
+        self.setAlignment(QtCore.Qt.AlignmentFlag.AlignTop | QtCore.Qt.AlignmentFlag.AlignHCenter)
         self.setMargin(0)
         self.setAcceptDrops(drops)
         self.clicked.connect(self.open_release_page)
         self.related_images = []
-        self._pixmap_cache = pixmap_cache
         self.current_pixmap_key = None
 
+    def screen_changed(self, screen):
+        pixel_ratio = screen.devicePixelRatio()
+        log.debug('screen changed, pixel ratio %s', pixel_ratio)
+        if pixel_ratio != self.pixel_ratio:
+            self.pixel_ratio = pixel_ratio
+            self._update_default_pixmaps()
+            if self.data:
+                self.set_data(self.data, force=True, has_common_images=self.has_common_images)
+            else:
+                self.setPixmap(self.shadow)
+
+    def _update_default_pixmaps(self):
+        w, h = self.scaled(THUMBNAIL_WIDTH, THUMBNAIL_WIDTH)
+        self.shadow = self._load_cached_default_pixmap(':/images/CoverArtShadow.png', w, h)
+        self.file_missing_pixmap = self._load_cached_default_pixmap(':/images/image-missing.png', w, h)
+
+    def _load_cached_default_pixmap(self, pixmap_path, w, h):
+        key = hash((pixmap_path, self.pixel_ratio))
+        try:
+            pixmap = self._pixmap_cache[key]
+        except KeyError:
+            pixmap = QtGui.QPixmap(pixmap_path)
+            pixmap = pixmap.scaled(w, h, QtCore.Qt.AspectRatioMode.KeepAspectRatio, QtCore.Qt.TransformationMode.SmoothTransformation)
+            pixmap.setDevicePixelRatio(self.pixel_ratio)
+            self._pixmap_cache[key] = pixmap
+        return pixmap
+
     def __eq__(self, other):
         if len(self.data) or len(other.data):
             return self.current_pixmap_key == other.current_pixmap_key
@@ -141,20 +178,6 @@ class CoverArtThumbnail(ActiveLabel):
     def show(self):
         self.set_data(self.data, True)
 
-    def decorate_cover(self, pixmap):
-        offx, offy, w, h = self.scaled(1, 1, 121, 121)
-        cover = QtGui.QPixmap(self.shadow)
-        pixmap = pixmap.scaled(w, h, QtCore.Qt.KeepAspectRatio, QtCore.Qt.SmoothTransformation)
-        pixmap.setDevicePixelRatio(self.pixel_ratio)
-        painter = QtGui.QPainter(cover)
-        bgcolor = QtGui.QColor.fromRgb(0, 0, 0, 128)
-        painter.fillRect(QtCore.QRectF(offx, offy, w, h), bgcolor)
-        x = offx + (w - pixmap.width()) // 2
-        y = offy + (h - pixmap.height()) // 2
-        painter.drawPixmap(x, y, pixmap)
-        painter.end()
-        return cover
-
     def set_data(self, data, force=False, has_common_images=True):
         if not force and self.data == data and self.has_common_images == has_common_images:
             return
@@ -173,72 +196,118 @@ class CoverArtThumbnail(ActiveLabel):
         if len(self.data) == 1:
             has_common_images = True
 
-        w, h, displacements = self.scaled(128, 128, 20)
-        key = hash(tuple(sorted(self.data, key=lambda x: x.types_as_string())) + (has_common_images,))
+        key = hash(tuple(sorted(self.data, key=lambda x: x.types_as_string())) + (has_common_images, self.pixel_ratio))
         try:
             pixmap = self._pixmap_cache[key]
         except KeyError:
             if len(self.data) == 1:
                 pixmap = QtGui.QPixmap()
-                pixmap.loadFromData(self.data[0].data)
-                pixmap = self.decorate_cover(pixmap)
+                try:
+                    pixmap.loadFromData(self.data[0].data)
+                    pixmap = self.decorate_cover(pixmap)
+                except CoverArtImageIOError:
+                    pixmap = self.file_missing_pixmap
             else:
-                limited = len(self.data) > MAX_COVERS_TO_STACK
-                if limited:
-                    data_to_paint = data[:MAX_COVERS_TO_STACK - 1]
-                    offset = displacements * len(data_to_paint)
-                else:
-                    data_to_paint = data
-                    offset = displacements * (len(data_to_paint) - 1)
-                stack_width, stack_height = (w + offset, h + offset)
-                pixmap = QtGui.QPixmap(stack_width, stack_height)
-                bgcolor = self.palette().color(QtGui.QPalette.Window)
-                painter = QtGui.QPainter(pixmap)
-                painter.fillRect(QtCore.QRectF(0, 0, stack_width, stack_height), bgcolor)
-                cx = stack_width - w // 2
-                cy = h // 2
-                if limited:
-                    x, y = (cx - self.shadow.width() // 2, cy - self.shadow.height() // 2)
-                    for i in range(3):
-                        painter.drawPixmap(x, y, self.shadow)
-                        x -= displacements // 3
-                        y += displacements // 3
-                    cx -= displacements
-                    cy += displacements
-                else:
-                    cx = stack_width - w // 2
-                    cy = h // 2
-                for image in reversed(data_to_paint):
-                    if isinstance(image, QtGui.QPixmap):
-                        thumb = image
-                    else:
-                        thumb = QtGui.QPixmap()
-                        thumb.loadFromData(image.data)
-                    thumb = self.decorate_cover(thumb)
-                    x, y = (cx - thumb.width() // 2, cy - thumb.height() // 2)
-                    painter.drawPixmap(x, y, thumb)
-                    cx -= displacements
-                    cy += displacements
-                if not has_common_images:
-                    color = QtGui.QColor("darkgoldenrod")
-                    border_length = 10
-                    for k in range(border_length):
-                        color.setAlpha(255 - k * 255 // border_length)
-                        painter.setPen(color)
-                        painter.drawLine(x, y - k - 1, x + 121 + k + 1, y - k - 1)
-                        painter.drawLine(x + 121 + k + 2, y - 1 - k, x + 121 + k + 2, y + 121 + 4)
-                    for k in range(5):
-                        bgcolor.setAlpha(80 + k * 255 // 7)
-                        painter.setPen(bgcolor)
-                        painter.drawLine(x + 121 + 2, y + 121 + 2 + k, x + 121 + border_length + 2, y + 121 + 2 + k)
-                painter.end()
-                pixmap = pixmap.scaled(w, h, QtCore.Qt.KeepAspectRatio, QtCore.Qt.SmoothTransformation)
+                pixmap = self.render_cover_stack(self.data, has_common_images)
             self._pixmap_cache[key] = pixmap
 
-        pixmap.setDevicePixelRatio(self.pixel_ratio)
         self.setPixmap(pixmap)
         self.current_pixmap_key = key
 
+    def decorate_cover(self, pixmap):
+        offx = offy = 1
+        w = h = COVERART_WIDTH
+        cover = QtGui.QPixmap(self.shadow)
+        cover.setDevicePixelRatio(self.pixel_ratio)
+        pixmap = pixmap.scaled(*self.scaled(w, h), QtCore.Qt.AspectRatioMode.KeepAspectRatio, QtCore.Qt.TransformationMode.SmoothTransformation)
+        pixmap.setDevicePixelRatio(self.pixel_ratio)
+        painter = QtGui.QPainter(cover)
+        bgcolor = QtGui.QColor.fromRgb(0, 0, 0, 128)
+        painter.fillRect(QtCore.QRectF(offx, offy, w, h), bgcolor)
+        self._draw_centered(painter, pixmap, w, h, offx, offy)
+        painter.end()
+        return cover
+
+    @staticmethod
+    def _draw_centered(painter, pixmap, width, height, offset_x=0, offset_y=0):
+        pixel_ratio = pixmap.devicePixelRatio()
+        x = int(offset_x + (width - pixmap.width() / pixel_ratio) // 2)
+        y = int(offset_y + (height - pixmap.height() / pixel_ratio) // 2)
+        painter.drawPixmap(x, y, pixmap)
+
+    def render_cover_stack(self, data, has_common_images):
+        w = h = THUMBNAIL_WIDTH
+        displacements = 20
+        limited = len(data) > MAX_COVERS_TO_STACK
+        if limited:
+            data_to_paint = data[:MAX_COVERS_TO_STACK - 1]
+            offset = displacements * len(data_to_paint)
+        else:
+            data_to_paint = data
+            offset = displacements * (len(data_to_paint) - 1)
+        stack_width, stack_height = (w + offset, h + offset)
+        pixmap = QtGui.QPixmap(*self.scaled(stack_width, stack_height))
+        pixmap.setDevicePixelRatio(self.pixel_ratio)
+        bgcolor = self.palette().color(QtGui.QPalette.ColorRole.Window)
+        painter = QtGui.QPainter(pixmap)
+        painter.fillRect(QtCore.QRectF(0, 0, stack_width, stack_height), bgcolor)
+        cx = stack_width - w // 2
+        cy = h // 2
+
+        def calculate_cover_coordinates(pixmap, cx, cy):
+            pixel_ratio = pixmap.devicePixelRatio()
+            x = int(cx - pixmap.width() / pixel_ratio // 2)
+            y = int(cy - pixmap.height() / pixel_ratio // 2)
+            return x, y
+
+        if limited:
+            # Draw the default background three times to indicate that there are more
+            # covers than the ones displayed
+            x, y = calculate_cover_coordinates(self.shadow, cx, cy)
+            for i in range(3):
+                painter.drawPixmap(x, y, self.shadow)
+                x -= displacements // 3
+                y += displacements // 3
+            cx -= displacements
+            cy += displacements
+        else:
+            cx = stack_width - w // 2
+            cy = h // 2
+        for image in reversed(data_to_paint):
+            if isinstance(image, QtGui.QPixmap):
+                thumb = image
+            else:
+                thumb = QtGui.QPixmap()
+                try:
+                    thumb.loadFromData(image.data)
+                except CoverArtImageIOError:
+                    thumb = self.file_missing_pixmap
+            thumb = self.decorate_cover(thumb)
+            x, y = calculate_cover_coordinates(thumb, cx, cy)
+            painter.drawPixmap(x, y, thumb)
+            cx -= displacements
+            cy += displacements
+        if not has_common_images:
+            # Draw a golden highlight around the first cover to indicate that
+            # images are not common to all selected items
+            color = QtGui.QColor("darkgoldenrod")
+            border_length = 10
+            for k in range(border_length):
+                color.setAlpha(255 - k * 255 // border_length)
+                painter.setPen(color)
+                x_offset = x + COVERART_WIDTH + k
+                # Horizontal line above the cover
+                painter.drawLine(x, y - k - 1, x_offset + 1, y - k - 1)
+                # Vertical line right of the cover
+                painter.drawLine(x_offset + 2, y - 1 - k, x_offset + 2, y + COVERART_WIDTH + 4)
+            # A bit of shadow
+            for k in range(5):
+                bgcolor.setAlpha(80 + k * 255 // 7)
+                painter.setPen(bgcolor)
+                painter.drawLine(x + COVERART_WIDTH + 2, y + COVERART_WIDTH + 2 + k, x + COVERART_WIDTH + border_length + 2, y + COVERART_WIDTH + 2 + k)
+        painter.end()
+        return pixmap.scaled(*self.scaled(w, h), QtCore.Qt.AspectRatioMode.KeepAspectRatio, QtCore.Qt.TransformationMode.SmoothTransformation)
+
     def set_metadata(self, metadata):
         data = None
         self.related_images = []
@@ -312,13 +381,13 @@ class CoverArtBox(QtWidgets.QGroupBox):
         self.item = None
         self.pixmap_cache = LRUCache(40)
         self.cover_art_label = QtWidgets.QLabel('')
-        self.cover_art_label.setAlignment(QtCore.Qt.AlignTop | QtCore.Qt.AlignHCenter)
+        self.cover_art_label.setAlignment(QtCore.Qt.AlignmentFlag.AlignTop | QtCore.Qt.AlignmentFlag.AlignHCenter)
         self.cover_art = CoverArtThumbnail(False, True, self.pixmap_cache, parent)
         self.cover_art.image_dropped.connect(self.fetch_remote_image)
-        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
+        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
         self.orig_cover_art_label = QtWidgets.QLabel('')
         self.orig_cover_art = CoverArtThumbnail(False, False, self.pixmap_cache, parent)
-        self.orig_cover_art_label.setAlignment(QtCore.Qt.AlignTop | QtCore.Qt.AlignHCenter)
+        self.orig_cover_art_label.setAlignment(QtCore.Qt.AlignmentFlag.AlignTop | QtCore.Qt.AlignmentFlag.AlignHCenter)
         self.show_details_button = QtWidgets.QPushButton(_('Show more details'), self)
         self.layout.addWidget(self.cover_art_label)
         self.layout.addWidget(self.cover_art)
@@ -411,7 +480,7 @@ class CoverArtBox(QtWidgets.QGroupBox):
                                        parse_response_type=None, queryargs=queryargs,
                                        priority=True, important=True)
         elif url.scheme() == 'file':
-            path = os.path.normpath(os.path.realpath(url.toLocalFile().rstrip("\0")))
+            path = normpath(url.toLocalFile().rstrip("\0"))
             if path and os.path.exists(path):
                 with open(path, 'rb') as f:
                     data = f.read()
@@ -425,7 +494,7 @@ class CoverArtBox(QtWidgets.QGroupBox):
             return
 
         data = bytes(data)
-        mime = reply.header(QtNetwork.QNetworkRequest.ContentTypeHeader)
+        mime = reply.header(QtNetwork.QNetworkRequest.KnownHeaders.ContentTypeHeader)
         # Some sites return a mime type with encoding like "image/jpeg; charset=UTF-8"
         mime = mime.split(';')[0]
         url_query = QtCore.QUrlQuery(url.query())
@@ -440,12 +509,12 @@ class CoverArtBox(QtWidgets.QGroupBox):
                 pass
         if url_query.hasQueryItem("imgurl"):
             # This may be a google images result, try to get the URL which is encoded in the query
-            url = QtCore.QUrl(url_query.queryItemValue("imgurl", QtCore.QUrl.FullyDecoded))
+            url = QtCore.QUrl(url_query.queryItemValue("imgurl", QtCore.QUrl.ComponentFormattingOption.FullyDecoded))
             log.debug('Possible Google images result, trying to fetch imgurl=%s', url.toString())
             self.fetch_remote_image(url)
         elif url_query.hasQueryItem("mediaurl"):
             # Bing uses mediaurl
-            url = QtCore.QUrl(url_query.queryItemValue("mediaurl", QtCore.QUrl.FullyDecoded))
+            url = QtCore.QUrl(url_query.queryItemValue("mediaurl", QtCore.QUrl.ComponentFormattingOption.FullyDecoded))
             log.debug('Possible Bing images result, trying to fetch imgurl=%s', url.toString())
             self.fetch_remote_image(url)
         else:
