@@ -4,7 +4,7 @@
 #
 # Copyright (C) 2006-2008, 2011 Lukáš Lalinský
 # Copyright (C) 2008-2009 Nikolai Prokoschenko
-# Copyright (C) 2008-2009, 2018-2021 Philipp Wolfer
+# Copyright (C) 2008-2009, 2018-2022 Philipp Wolfer
 # Copyright (C) 2011 Pavan Chander
 # Copyright (C) 2011-2012, 2019 Wieland Hoffmann
 # Copyright (C) 2011-2013 Michael Wiencek
@@ -13,7 +13,7 @@
 # Copyright (C) 2016-2017 Sambhav Kothari
 # Copyright (C) 2017 Suhas
 # Copyright (C) 2018 Vishal Choudhary
-# Copyright (C) 2021 Bob Swift
+# Copyright (C) 2021-2022 Bob Swift
 # Copyright (C) 2021 Gabriel Ferreira
 #
 # This program is free software; you can redistribute it and/or
@@ -57,7 +57,6 @@ from picard.ui import (
 from picard.ui.options import (  # noqa: F401 # pylint: disable=unused-import
     OptionsCheckError,
     _pages as page_classes,
-    acousticbrainz,
     advanced,
     cdlookup,
     cover,
@@ -106,7 +105,7 @@ class OptionsDialog(PicardDialog, SingletonDialog):
                 self.page_to_item[page.NAME] = item
                 self.ui.pages_stack.addWidget(page)
             else:
-                item.setFlags(QtCore.Qt.ItemIsEnabled)
+                item.setFlags(QtCore.Qt.ItemFlag.ItemIsEnabled)
             self.add_pages(page.NAME, default_page, item)
             if page.NAME == default_page:
                 self.default_item = item
@@ -116,8 +115,8 @@ class OptionsDialog(PicardDialog, SingletonDialog):
 
     def __init__(self, default_page=None, parent=None):
         super().__init__(parent)
-        self.setWindowModality(QtCore.Qt.ApplicationModal)
-        self.setAttribute(QtCore.Qt.WA_DeleteOnClose)
+        self.setWindowModality(QtCore.Qt.WindowModality.ApplicationModal)
+        self.setAttribute(QtCore.Qt.WidgetAttribute.WA_DeleteOnClose)
 
         from picard.ui.ui_options import Ui_Dialog
         self.ui = Ui_Dialog()
@@ -130,11 +129,11 @@ class OptionsDialog(PicardDialog, SingletonDialog):
 
         ok = StandardButton(StandardButton.OK)
         ok.setText(_("Make It So!"))
-        self.ui.buttonbox.addButton(ok, QtWidgets.QDialogButtonBox.AcceptRole)
-        self.ui.buttonbox.addButton(StandardButton(StandardButton.CANCEL), QtWidgets.QDialogButtonBox.RejectRole)
-        self.ui.buttonbox.addButton(StandardButton(StandardButton.HELP), QtWidgets.QDialogButtonBox.HelpRole)
-        self.ui.buttonbox.addButton(self.ui.reset_all_button, QtWidgets.QDialogButtonBox.ActionRole)
-        self.ui.buttonbox.addButton(self.ui.reset_button, QtWidgets.QDialogButtonBox.ActionRole)
+        self.ui.buttonbox.addButton(ok, QtWidgets.QDialogButtonBox.ButtonRole.AcceptRole)
+        self.ui.buttonbox.addButton(StandardButton(StandardButton.CANCEL), QtWidgets.QDialogButtonBox.ButtonRole.RejectRole)
+        self.ui.buttonbox.addButton(StandardButton(StandardButton.HELP), QtWidgets.QDialogButtonBox.ButtonRole.HelpRole)
+        self.ui.buttonbox.addButton(self.ui.reset_all_button, QtWidgets.QDialogButtonBox.ButtonRole.ActionRole)
+        self.ui.buttonbox.addButton(self.ui.reset_button, QtWidgets.QDialogButtonBox.ButtonRole.ActionRole)
 
         self.ui.buttonbox.accepted.connect(self.accept)
         self.ui.buttonbox.rejected.connect(self.reject)
@@ -144,7 +143,7 @@ class OptionsDialog(PicardDialog, SingletonDialog):
 
         self.ui.attached_profiles_button = QtWidgets.QPushButton(_("Attached Profiles"))
         self.ui.attached_profiles_button.setToolTip(_("Show which profiles are attached to the options on this page"))
-        self.ui.buttonbox.addButton(self.ui.attached_profiles_button, QtWidgets.QDialogButtonBox.ActionRole)
+        self.ui.buttonbox.addButton(self.ui.attached_profiles_button, QtWidgets.QDialogButtonBox.ButtonRole.ActionRole)
         self.ui.attached_profiles_button.clicked.connect(self.show_attached_profiles_dialog)
 
         config = get_config()
@@ -175,17 +174,15 @@ class OptionsDialog(PicardDialog, SingletonDialog):
         self.restoreWindowState()
         self.finished.connect(self.saveWindowState)
 
-        for page in self.pages:
-            try:
-                page.load()
-            except Exception:
-                log.exception('Failed loading options page %r', page)
-                self.disable_page(page.NAME)
+        self.load_all_pages()
         self.ui.pages_tree.setCurrentItem(self.default_item)
 
         self.profile_page = self.get_page('profiles')
         self.profile_page.signal_refresh.connect(self.update_from_profile_changes)
 
+        self.maintenance_page = self.get_page('maintenance')
+        self.maintenance_page.signal_reload.connect(self.load_all_pages)
+
         self.first_enter = True
         self.installEventFilter(self)
 
@@ -193,6 +190,14 @@ class OptionsDialog(PicardDialog, SingletonDialog):
         current_page = self.item_to_page[self.ui.pages_tree.currentItem()]
         self.set_profiles_button_and_highlight(current_page)
 
+    def load_all_pages(self):
+        for page in self.pages:
+            try:
+                page.load()
+            except Exception:
+                log.exception('Failed loading options page %r', page)
+                self.disable_page(page.NAME)
+
     def page_has_profile_options(self, page):
         try:
             name = page.PARENT if page.PARENT in UserProfileGroups.SETTINGS_GROUPS else page.NAME
@@ -208,11 +213,11 @@ class OptionsDialog(PicardDialog, SingletonDialog):
         page = self.item_to_page[items[0]]
         if not self.page_has_profile_options(page):
             message_box = QtWidgets.QMessageBox(self)
-            message_box.setIcon(QtWidgets.QMessageBox.Information)
-            message_box.setWindowModality(QtCore.Qt.WindowModal)
+            message_box.setIcon(QtWidgets.QMessageBox.Icon.Information)
+            message_box.setWindowModality(QtCore.Qt.WindowModality.WindowModal)
             message_box.setWindowTitle(window_title)
             message_box.setText(_("The options on this page are not currently available to be managed using profiles."))
-            message_box.setStandardButtons(QtWidgets.QMessageBox.Ok)
+            message_box.setStandardButtons(QtWidgets.QMessageBox.StandardButton.Ok)
             return message_box.exec_()
 
         override_profiles = self.profile_page._clean_and_get_all_profiles()
@@ -293,7 +298,7 @@ class OptionsDialog(PicardDialog, SingletonDialog):
         """Process selected events.
         """
         evtype = event.type()
-        if evtype == QtCore.QEvent.Enter:
+        if evtype == QtCore.QEvent.Type.Enter:
             if self.first_enter:
                 self.first_enter = False
                 if self.tagger and self.tagger.window.script_editor_dialog is not None:
@@ -421,12 +426,12 @@ class OptionsDialog(PicardDialog, SingletonDialog):
 
     def _show_dialog(self, msg, function):
         message_box = QtWidgets.QMessageBox(self)
-        message_box.setIcon(QtWidgets.QMessageBox.Warning)
-        message_box.setWindowModality(QtCore.Qt.WindowModal)
+        message_box.setIcon(QtWidgets.QMessageBox.Icon.Warning)
+        message_box.setWindowModality(QtCore.Qt.WindowModality.WindowModal)
         message_box.setWindowTitle(_("Confirm Reset"))
         message_box.setText(_("Are you sure?") + "\n\n" + msg)
-        message_box.setStandardButtons(QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No)
-        if message_box.exec_() == QtWidgets.QMessageBox.Yes:
+        message_box.setStandardButtons(QtWidgets.QMessageBox.StandardButton.Yes | QtWidgets.QMessageBox.StandardButton.No)
+        if message_box.exec_() == QtWidgets.QMessageBox.StandardButton.Yes:
             function()
 
 
@@ -439,7 +444,7 @@ class AttachedProfilesDialog(PicardDialog):
         self.option_group = option_group
         self.ui = Ui_AttachedProfilesDialog()
         self.ui.setupUi(self)
-        self.ui.buttonBox.addButton(StandardButton(StandardButton.CLOSE), QtWidgets.QDialogButtonBox.RejectRole)
+        self.ui.buttonBox.addButton(StandardButton(StandardButton.CLOSE), QtWidgets.QDialogButtonBox.ButtonRole.RejectRole)
         self.ui.buttonBox.rejected.connect(self.close_window)
 
         config = get_config()
