@@ -4,9 +4,9 @@
 #
 # Copyright (C) 2007 Lukáš Lalinský
 # Copyright (C) 2009 Carlin Mangar
-# Copyright (C) 2009, 2018-2020 Philipp Wolfer
+# Copyright (C) 2009, 2018-2022 Philipp Wolfer
 # Copyright (C) 2011-2013 Michael Wiencek
-# Copyright (C) 2013, 2015, 2018-2019 Laurent Monin
+# Copyright (C) 2013, 2015, 2018-2021 Laurent Monin
 # Copyright (C) 2013, 2017 Sophist-UK
 # Copyright (C) 2014 Shadab Zafar
 # Copyright (C) 2015, 2017 Wieland Hoffmann
@@ -31,8 +31,10 @@
 
 
 from functools import partial
+from html import escape
 from operator import attrgetter
 import os.path
+import re
 
 from PyQt5 import (
     QtCore,
@@ -199,7 +201,7 @@ class PluginTreeWidgetItem(HashableTreeWidgetItem):
 
     @property
     def plugin(self):
-        return self.data(COLUMN_NAME, QtCore.Qt.UserRole)
+        return self.data(COLUMN_NAME, QtCore.Qt.ItemDataRole.UserRole)
 
     def enable(self, boolean, greyout=None):
         if boolean is not None:
@@ -225,7 +227,7 @@ class PluginsOptionsPage(OptionsPage):
         ListOption("setting", "enabled_plugins", []),
         Option("persist", "plugins_list_state", QtCore.QByteArray()),
         Option("persist", "plugins_list_sort_section", 0),
-        Option("persist", "plugins_list_sort_order", QtCore.Qt.AscendingOrder),
+        Option("persist", "plugins_list_sort_order", QtCore.Qt.SortOrder.AscendingOrder),
     ]
 
     def __init__(self, parent=None):
@@ -256,7 +258,7 @@ class PluginsOptionsPage(OptionsPage):
         self._preserve_selected = None
 
     def items(self):
-        iterator = QTreeWidgetItemIterator(self.ui.plugins, QTreeWidgetItemIterator.All)
+        iterator = QTreeWidgetItemIterator(self.ui.plugins, QTreeWidgetItemIterator.IteratorFlag.All)
         while iterator.value():
             item = iterator.value()
             iterator += 1
@@ -302,7 +304,7 @@ class PluginsOptionsPage(OptionsPage):
         return bool(plugin.module_name in config.setting["enabled_plugins"])
 
     def available_plugins_name_version(self):
-        return dict([(p.module_name, p.version) for p in self.manager.available_plugins])
+        return {p.module_name: p.version for p in self.manager.available_plugins}
 
     def installable_plugins(self):
         if self.manager.available_plugins is not None:
@@ -350,10 +352,10 @@ class PluginsOptionsPage(OptionsPage):
         self._user_interaction(True)
         header = self.ui.plugins.header()
         header.setStretchLastSection(False)
-        header.setSectionResizeMode(QtWidgets.QHeaderView.Fixed)
-        header.setSectionResizeMode(COLUMN_NAME, QtWidgets.QHeaderView.Stretch)
-        header.setSectionResizeMode(COLUMN_VERSION, QtWidgets.QHeaderView.ResizeToContents)
-        header.setSectionResizeMode(COLUMN_ACTIONS, QtWidgets.QHeaderView.ResizeToContents)
+        header.setSectionResizeMode(QtWidgets.QHeaderView.ResizeMode.Fixed)
+        header.setSectionResizeMode(COLUMN_NAME, QtWidgets.QHeaderView.ResizeMode.Stretch)
+        header.setSectionResizeMode(COLUMN_VERSION, QtWidgets.QHeaderView.ResizeMode.ResizeToContents)
+        header.setSectionResizeMode(COLUMN_ACTIONS, QtWidgets.QHeaderView.ResizeMode.ResizeToContents)
 
     def _remove_all(self):
         for item in self.items():
@@ -460,10 +462,10 @@ class PluginsOptionsPage(OptionsPage):
             self,
             _("Uninstall plugin '%s'?") % plugin.name,
             _("Do you really want to uninstall the plugin '%s' ?") % plugin.name,
-            QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
-            QtWidgets.QMessageBox.No
+            QtWidgets.QMessageBox.StandardButton.Yes | QtWidgets.QMessageBox.StandardButton.No,
+            QtWidgets.QMessageBox.StandardButton.No
         )
-        if buttonReply == QtWidgets.QMessageBox.Yes:
+        if buttonReply == QtWidgets.QMessageBox.StandardButton.Yes:
             self.manager.remove_plugin(plugin.module_name, with_update=True)
 
     def update_plugin_item(self, item, plugin,
@@ -475,7 +477,7 @@ class PluginsOptionsPage(OptionsPage):
         if item is None:
             item = PluginTreeWidgetItem(self.ui.plugins)
         if plugin is not None:
-            item.setData(COLUMN_NAME, QtCore.Qt.UserRole, plugin)
+            item.setData(COLUMN_NAME, QtCore.Qt.ItemDataRole.UserRole, plugin)
         else:
             plugin = item.plugin
         if new_version is not None:
@@ -599,16 +601,33 @@ class PluginsOptionsPage(OptionsPage):
         if plugin.description:
             text.append(plugin.description + "<hr width='90%'/>")
         infos = [
-            (_("Name"), plugin.name),
-            (_("Authors"), plugin.author),
+            (_("Name"), escape(plugin.name)),
+            (_("Authors"), self.link_authors(plugin.author)),
             (_("License"), plugin.license),
-            (_("Files"), plugin.files_list),
+            (_("Files"), escape(plugin.files_list)),
         ]
         for label, value in infos:
             if value:
                 text.append("<b>{0}:</b> {1}".format(label, value))
         self.ui.details.setText("<p>{0}</p>".format("<br/>\n".join(text)))
 
+    @staticmethod
+    def link_authors(authors):
+        formatted_authors = []
+        re_author = re.compile(r"(?P<author>.*?)\s*<(?P<email>.*?@.*?)>")
+        for author in authors.split(','):
+            author = author.strip()
+            match = re_author.fullmatch(author)
+            if match:
+                author_str = '<a href="mailto:{email}">{author}</a>'.format(
+                    email=escape(match['email']),
+                    author=escape(match['author']),
+                )
+                formatted_authors.append(author_str)
+            else:
+                formatted_authors.append(escape(author))
+        return ', '.join(formatted_authors)
+
     def change_details(self):
         item = self.selected_item()
         if item:
@@ -644,8 +663,8 @@ class PluginsOptionsPage(OptionsPage):
             msgbox = QtWidgets.QMessageBox(self)
             msgbox.setText(_("The plugin '%s' could not be downloaded.") % plugin.module_name)
             msgbox.setInformativeText(_("Please try again later."))
-            msgbox.setStandardButtons(QtWidgets.QMessageBox.Ok)
-            msgbox.setDefaultButton(QtWidgets.QMessageBox.Ok)
+            msgbox.setStandardButtons(QtWidgets.QMessageBox.StandardButton.Ok)
+            msgbox.setDefaultButton(QtWidgets.QMessageBox.StandardButton.Ok)
             msgbox.exec_()
             log.error("Error occurred while trying to download the plugin: '%s'" % plugin.module_name)
             return
@@ -665,11 +684,11 @@ class PluginsOptionsPage(OptionsPage):
         return ["text/uri-list"]
 
     def dragEnterEvent(self, event):
-        event.setDropAction(QtCore.Qt.CopyAction)
+        event.setDropAction(QtCore.Qt.DropAction.CopyAction)
         event.accept()
 
     def dropEvent(self, event):
-        for path in [os.path.normpath(u.toLocalFile()) for u in event.mimeData().urls()]:
+        for path in (os.path.normpath(u.toLocalFile()) for u in event.mimeData().urls()):
             self.manager.install_plugin(path)
 
 
