@@ -3,7 +3,7 @@
 # Picard, the next-generation MusicBrainz tagger
 #
 # Copyright (C) 2006 Lukáš Lalinský
-# Copyright (C) 2019-2020 Philipp Wolfer
+# Copyright (C) 2019-2020, 2022 Philipp Wolfer
 # Copyright (C) 2020-2021 Laurent Monin
 #
 # This program is free software; you can redistribute it and/or
@@ -48,7 +48,7 @@ class ElidedLabel(QtWidgets.QLabel):
         # text does not properly fit into width(), as a workaround subtract
         # 2 pixels from the available width.
         elided_label = metrics.elidedText(self._full_label,
-                                          QtCore.Qt.ElideRight,
+                                          QtCore.Qt.TextElideMode.ElideRight,
                                           self.width() - 2)
         super().setText(elided_label)
         if self._full_label and elided_label != self._full_label:
@@ -69,12 +69,12 @@ class ActiveLabel(QtWidgets.QLabel):
     def setActive(self, active):
         self.active = active
         if self.active:
-            self.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
+            self.setCursor(QtGui.QCursor(QtCore.Qt.CursorShape.PointingHandCursor))
         else:
             self.setCursor(QtGui.QCursor())
 
     def mouseReleaseEvent(self, event):
-        if self.active and event.button() == QtCore.Qt.LeftButton:
+        if self.active and event.button() == QtCore.Qt.MouseButton.LeftButton:
             self.clicked.emit()
 
 
@@ -102,12 +102,19 @@ class Popover(QtWidgets.QFrame):
 
     def __init__(self, parent, position='bottom'):
         super().__init__(parent)
-        self.setWindowFlags(QtCore.Qt.Popup | QtCore.Qt.FramelessWindowHint)
+        self.setWindowFlags(QtCore.Qt.WindowType.Popup | QtCore.Qt.WindowType.FramelessWindowHint)
         self.position = position
+        app = QtCore.QCoreApplication.instance()
+        self._is_wayland = app.is_wayland
+        self._main_window = app.window
 
     def show(self):
-        super().show()
+        super().show()  # show so that the geometry gets known
         self.update_position()
+        # on Wayland position updates only work if widget gets shown
+        if self._is_wayland:
+            super().hide()
+            super().show()
 
     def update_position(self):
         parent = self.parent()
@@ -116,16 +123,29 @@ class Popover(QtWidgets.QFrame):
             y = -self.height()
         else:  # bottom
             y = parent.height()
-        pos = parent.mapToGlobal(QtCore.QPoint(x, y))
-        screen_number = QtWidgets.QApplication.desktop().screenNumber()
-        screen = QtGui.QGuiApplication.screens()[screen_number]
-        screen_size = screen.availableVirtualSize()
-        if pos.x() < 0:
-            pos.setX(0)
+        pos = QtCore.QPoint(int(x), int(y))
+        pos = parent.mapToGlobal(pos)
+        if not self._is_wayland:
+            # Attempt to keep the popover fully visible on screen.
+            min_pos = QtCore.QPoint(0, 0)
+            screen_number = QtWidgets.QApplication.desktop().screenNumber()
+            screen = QtGui.QGuiApplication.screens()[screen_number]
+            screen_size = screen.availableVirtualSize()
+        else:
+            # The full screen size is not known on Wayland, but we can ensure
+            # the popover stays inside the app window boundary.
+            min_pos = self._main_window.pos()
+            window_size = self._main_window.size()
+            screen_size = QtCore.QSize(
+                min_pos.x() + window_size.width(),
+                min_pos.y() + window_size.height(),
+            )
+        if pos.x() < min_pos.x():
+            pos.setX(min_pos.x())
         if pos.x() + self.width() > screen_size.width():
             pos.setX(screen_size.width() - self.width())
-        if pos.y() < 0:
-            pos.setY(0)
+        if pos.y() < min_pos.y():
+            pos.setY(min_pos.y())
         if pos.y() + self.height() > screen_size.height():
             pos.setY(screen_size.height() - self.height())
         self.move(pos)
@@ -140,11 +160,11 @@ class SliderPopover(Popover):
         super().__init__(parent, position)
         vbox = QtWidgets.QVBoxLayout(self)
         self.label = QtWidgets.QLabel(label, self)
-        self.label.setAlignment(QtCore.Qt.AlignCenter)
+        self.label.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
         vbox.addWidget(self.label)
 
         self.slider = ClickableSlider(self)
-        self.slider.setOrientation(QtCore.Qt.Horizontal)
-        self.slider.setValue(value)
+        self.slider.setOrientation(QtCore.Qt.Orientation.Horizontal)
+        self.slider.setValue(int(value))
         self.slider.valueChanged.connect(self.value_changed)
         vbox.addWidget(self.slider)
