@@ -1,8 +1,15 @@
 # -*- coding: utf-8 -*-
 #
 # Picard, the next-generation MusicBrainz tagger
+#
 # Copyright (C) 2004 Robert Kaye
 # Copyright (C) 2006 Lukáš Lalinský
+# Copyright (C) 2014 Sophist-UK
+# Copyright (C) 2014, 2018, 2020-2021 Laurent Monin
+# Copyright (C) 2017 Sambhav Kothari
+# Copyright (C) 2018-2019, 2021 Philipp Wolfer
+# Copyright (C) 2020 Gabriel Ferreira
+# Copyright (C) 2020 Undearius
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,6 +25,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
+
 # This modules provides functionality for simplifying unicode strings.
 
 # The unicode character set (of over 1m codepoints and 24,000 characters) includes:
@@ -62,21 +70,17 @@
 
 import codecs
 from functools import partial
-import re
 import unicodedata
 
 from picard.util import sanitize_filename
 
-#########################  LATIN SIMPLIFICATION ###########################
+
+# LATIN SIMPLIFICATION
 # The translation tables for punctuation and latin combined-characters are taken from
 # http://unicode.org/repos/cldr/trunk/common/transforms/Latin-ASCII.xml
 # Various bugs and mistakes in this have been ironed out during testing.
 
 
-def _re_any(iterable):
-    return re.compile('([' + ''.join(iterable) + '])', re.UNICODE)
-
-
 _additional_compatibility = {
     "\u0276": "Œ",  # LATIN LETTER SMALL CAPITAL OE
     "\u1D00": "A",  # LATIN LETTER SMALL CAPITAL A
@@ -98,12 +102,12 @@ _additional_compatibility = {
     "\u00A0": " ",  # NO-BREAK SPACE
     "\u3000": " ",  # IDEOGRAPHIC SPACE (from ‹character-fallback›)
     "\u2033": "”",  # DOUBLE PRIME
+    "\uff0f": "/",  # FULLWIDTH SOLIDUS
 }
-_re_additional_compatibility = _re_any(_additional_compatibility.keys())
 
 
-def unicode_simplify_compatibility(string):
-    interim = _re_additional_compatibility.sub(lambda m: _additional_compatibility[m.group(0)], string)
+def unicode_simplify_compatibility(string, pathsave=False, win_compat=False):
+    interim = ''.join(_replace_char(_additional_compatibility, ch, pathsave, win_compat) for ch in string)
     return unicodedata.normalize("NFKC", interim)
 
 
@@ -175,15 +179,13 @@ _simplify_punctuation = {
     "\u226B": ">>",  # MUCH GREATER-THAN
     "\u2985": "((",  # LEFT WHITE PARENTHESIS
     "\u2986": "))",  # RIGHT WHITE PARENTHESIS
+    "\u2022": "-",  # BULLET
     "\u200B": "",  # Zero Width Space
 }
-_re_simplify_punctuation = _re_any(_simplify_punctuation.keys())
-_pathsave_simplify_punctuation = {k: sanitize_filename(v) for k, v in _simplify_punctuation.items()}
 
 
-def unicode_simplify_punctuation(string, pathsave=False):
-    punctuation = _pathsave_simplify_punctuation if pathsave else _simplify_punctuation
-    return _re_simplify_punctuation.sub(lambda m: punctuation[m.group(0)], string)
+def unicode_simplify_punctuation(string, pathsave=False, win_compat=False):
+    return ''.join(_replace_char(_simplify_punctuation, ch, pathsave, win_compat) for ch in string)
 
 
 _simplify_combinations = {
@@ -411,13 +413,21 @@ _simplify_combinations = {
     "\u0185": "h",  # LATIN SMALL LETTER TONE SIX
     "\u01BE": "ts",  # LATIN LETTER TS LIGATION (see http://unicode.org/notes/tn27/)
 }
-_re_simplify_combinations = _re_any(_simplify_combinations)
-_pathsave_simplify_combinations = {k: sanitize_filename(v) for k, v in _simplify_combinations.items()}
 
 
-def unicode_simplify_combinations(string, pathsave=False):
-    combinations = _pathsave_simplify_combinations if pathsave else _simplify_combinations
-    return _re_simplify_combinations.sub(lambda m: combinations[m.group(0)], string)
+def _replace_unicode_simplify_combinations(char, pathsave, win_compat):
+    result = _simplify_combinations.get(char)
+    if result is None:
+        return char
+    elif not pathsave:
+        return result
+    else:
+        return sanitize_filename(result, win_compat=win_compat)
+
+
+def unicode_simplify_combinations(string, pathsave=False, win_compat=False):
+    return ''.join(
+        _replace_unicode_simplify_combinations(c, pathsave, win_compat) for c in string)
 
 
 def unicode_simplify_accents(string):
@@ -435,15 +445,25 @@ def unaccent(string):
     return unicode_simplify_accents(string)
 
 
-def replace_non_ascii(string, repl="_", pathsave=False):
+def replace_non_ascii(string, repl="_", pathsave=False, win_compat=False):
     """Replace non-ASCII characters from ``string`` by ``repl``."""
-    interim = unicode_simplify_combinations(string, pathsave)
+    interim = unicode_simplify_combinations(string, pathsave, win_compat)
+    interim = unicode_simplify_punctuation(interim, pathsave, win_compat)
+    interim = unicode_simplify_compatibility(interim, pathsave, win_compat)
     interim = unicode_simplify_accents(interim)
-    interim = unicode_simplify_punctuation(interim, pathsave)
-    interim = unicode_simplify_compatibility(interim)
 
     def error_repl(e, repl="_"):
         return (repl, e.start + 1)
     codecs.register_error('repl', partial(error_repl, repl=repl))
     # Decoding and encoding to allow replacements
     return interim.encode('ascii', 'repl').decode('ascii')
+
+
+def _replace_char(map, ch, pathsave=False, win_compat=False):
+    try:
+        result = map[ch]
+        if ch != result and pathsave:
+            result = sanitize_filename(result, win_compat=win_compat)
+        return result
+    except KeyError:
+        return ch
